﻿/*
	*****************************
    *  VERSION NUMBERING SYSTEM *
    *****************************

    		X.XX.XX
	    	- -- --
    		| ||  |
        	| ||  |----- Increasing when Software modified (minor)
        	| ||
        	| ||-------- Increasing when Software changed (major)
        	| |
        	| |--------- Increasing when Firmware Changed
        	|            or important thing changed on software
        	|
        	|-----------   0 : Prototype Firmware version
        			   1 : Alpha
                       2 : Beta Test Version (Sample)
                       3 : Launch Version
                     4-9 : Increasing after Launched.

   -------------------------------------------------------------------------
   Date     By    Rev    Modifications / Affected Functions
   -------- --- -------  ---------------------------------------------------
   01/08/13 YC  2.01.01   Creation
   01/09/13 YC  2.01.02   Improve Data tranfer bug when using old PC(slow PC)
                            Packet_Xfer = 12 -> Packet_Xfer = 24
   01/31/13 YC  2.01.03   Added Error check rountine of the image info
                          Added timeout definition for the transfer to complete.
                          Modified SendCommand,ReadCommand functions.
   
   03/07/13 YC  3.00.00   Changed PID,VID
   03/26/13 YC  3.01.00   Remove USB high speed Check routine
   05/23/13 YC  3.01.01   Ignor "Scanner busy" message when system not ready.
   05/23/13 YC  3.01.02   Vacate the trash data in InEndpoint
   05/24/13 YC  3.01.03   Avoid duplication run
   05/24/18 YC  3.20.00   Windows 10 update issue (Ver 1803)
   02/12/19 YC  3.20.01   Data transmission timeout error issue.(#1902-01)
   03/04/19 YC  3.30.00   Application update for new hardware and device driver changes.(#1903-01)
 * 10/23/19 YC  3.40.00   Updated source code due to the issue with .NET framework 3.5 or higher on 64bit.(#1910-01)
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace M280_SampleApp
{
    class M280DEF
    {
        // Software Version
        public const string SWver = "3.40.00-Beta";
        public const string ProName = "M280 Sample App";

        // PID, VID
        // Application update for new hardware and device driver changes (#1903-01) -->
        public const ushort USB_PID = 0x0280;   //Previous version FX2
        public const ushort USB3_PID = 0x0282;  //new version FX3
        //<-- Application update for new hardware and device driver changes (#1903-01)
        public const ushort USB_VID = 0x28A6;

        // Define Command
        public const byte CMD_ST_CAP = 0xDA;
        public const byte CMD_SET_ILLUMIN = 0xE0;
        public const byte CMD_INIT_CAMERA = 0xE1;
        public const byte CMD_GET_STATE = 0xEA;
        public const byte CMD_GET_VERSION = 0xEB;
        public const byte CMD_GET_IMGINFO = 0xEE;
       
        // F/W version length
        public const byte MAX_CNUM_APP1 = 10;
        // Image Info size 20 byte
        public const byte IMGINFO_SIZE = 20;

        // Image download 
        //public const PixelFormat PixFormat = PixelFormat.Format24bppRgb;
        public const PixelFormat PixFormat = PixelFormat.Format16bppRgb565;
      
        //public const int Packet_Xfer = 12;    // Old 512 * 12 = 6144
        public const int Packet_Xfer = 24;      // New 512 * 24 = 12288
        public const int Image_Xsize = 1024;
        public const int Image_Ysize = 768;

        // Status 
        public const int STATUS_SIZE = 2;
        public const ushort stat_ReadyLED = 1 << 0;
        public const ushort stat_BusyLED = 1 << 1;
        public const ushort stat_CardDet = 1 << 2;
        public const ushort stat_CapDet = 1 << 3;
        public const ushort stat_CamInit = 1 << 4;
        public const ushort stat_SysBusy = 1 << 5;

        public const ushort stat_EngineEr = 1 << 7;
        public const ushort stat_EepromEr = 1 << 8;
        public const ushort stat_FPGAEr = 1 << 9;
        public const ushort StatusGetInterval = 100;  // 100mS

        // Others...
        public const ushort ON = 1;
        public const ushort OFF = 0;
        public const int TRANSFER_TIMEOUT = 1500;   //  Wait for the transfer to complete. 
    }

    class M280IMGDEF
    {
        public const int IMG_CARDSZ_X = 892;
        public const int IMG_CARDSZ_Y = 564;
        public const byte IMG_ANGLE = 0xFF / 2;
    }
}
